﻿//Copyright 2013, Andrew Polar
//Demo for Split and Merge algorithm

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Reflection;
using snowball;
using wordprocessing;
using splitandmerge;

namespace SplitAndMerge
{
    class EntryPointSAM
    {
        static void Main(string[] args)
        {
            //accuracy about 75%
            string[] folders = { @"C:\APOLAR\PATENTCORPUS128" };
            int numberOfExpectedClusters = 8;
            //Expected result. It is needed only for computation of the accuracy.
            int counter = 0;
            int[] nExpected = new int[128];
            for (int i = 0; i < 8; ++i)
            {
                for (int j = 0; j < 16; ++j)
                {
                    nExpected[counter++] = i;
                }
            }
            //end

            /*
            //accuracy about 40%
            string[] folders = { @"C:\APOLAR\PATENTCORPUS5000\Documents" };
            int numberOfExpectedClusters = 50;
            //Expected result. It is needed only for computation of the accuracy.
            int counter = 0;
            int[] nExpected = new int[5000];
            for (int i = 0; i < 50; ++i)
            {
                for (int j = 0; j < 100; ++j)
                {
                    nExpected[counter++] = i;
                }
            }
            //end
            */

            string masks = "*.txt";
            string databaseFolder = "data";
            string processedFilesList = "ProcessedFilesList.txt";
            string documentTermMatrix = "DocWordMatrix.dat";

            string path = Path.GetDirectoryName(Assembly.GetExecutingAssembly().Location);
            databaseFolder = Path.Combine(path, databaseFolder);
            processedFilesList = Path.Combine(databaseFolder, processedFilesList);
            documentTermMatrix = Path.Combine(databaseFolder, documentTermMatrix);

            //Find all files
            Console.WriteLine("Enumerating files ...");
            DateTime start = DateTime.Now;
            SimpleFileFinder sff = new SimpleFileFinder(folders, masks);
            sff.EnumerateFiles();
            int nNumberOfFiles = sff.GetNumberOfFiles();
            DateTime end = DateTime.Now;
            TimeSpan duration = end - start;
            double time = duration.Minutes * 60.0 + duration.Seconds + duration.Milliseconds / 1000.0;
            Console.WriteLine("{0} files found, during {1:####.00} seconds", nNumberOfFiles, time);
            //end

            //Making document-term matrix
            start = DateTime.Now;
            bool useStemming = true;  //This parameter enables or disables stemming.
            WordProcessing wp = new WordProcessing(databaseFolder, processedFilesList, documentTermMatrix, sff.m_foundFiles);
            wp.processFiles(useStemming);
            end = DateTime.Now;
            duration = end - start;
            time = duration.Minutes * 60.0 + duration.Seconds + duration.Milliseconds / 1000.0;
            Console.WriteLine("{0} files processed, during {1:####.00} seconds, {2} words in collection\n", sff.m_foundFiles.Count, time,
                wp.getNumberOfWords());
            //end 

            //Split and merge
            bool bUsePolarOrthogonalization = true;
            SAM sam = new SAM(databaseFolder, processedFilesList, documentTermMatrix, numberOfExpectedClusters, bUsePolarOrthogonalization);
            sam.MakeRandomLabels();

            for (int i = 0; i < 25; ++i)
            {
                sam.MakeClusters();
                sam.MakeSplitAndMergeStep();
                sam.ExecuteHAC();
                Console.Write("step {0}   \r", i + 1);
            }
            Console.WriteLine();

            double accuracy = sam.ComputeF_measure(nExpected, numberOfExpectedClusters);
            Console.WriteLine("Accuracy {0:####.00}\n", accuracy);
            sam.ShowClusteredFiles();
            sam.MakeTrainingSample(30);
            //end
        }
    }
}
