﻿//Copyright 2013, Andrew Polar
//Demo for Triple Clustering algorithm

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Reflection;
using helper;
using tripleclustering;

namespace TripleC
{
    class EntryPointTripleC
    {
        static void Main(string[] args)
        {
            //This is training sample obtained from Split and Merge algorithm. It is good only 
            //PATENTCORPUS128. For other data training sample has to be changed.
            //The training sample is recored in ProcessedFilesList.txt file in data directory.
            //Example of two lines from ProcessedFilesList.txt:
            //3, 0, C:\APOLAR\UTILITIES\PATENTCORPUS128\C399P7058343.txt
            //4, -1, C:\APOLAR\UTILITIES\PATENTCORPUS128\C399P7164866.txt
            //first number is index of processed file, second number is known category if >= 0 or
            //indicator that file to be categorized if < 0;
            //Make sure the training set is correct for the particular order of processed files.
            //It can be found in the file ProcessedFilesList.txt.
            //
            int N = 128;
            int[] nChart = new int[N];
            for (int i = 0; i < N; ++i)
            {
                nChart[i] = -1;
            }

            //This is training sample. Line nChart[16] = 1; nChart[21] = 1; nChart[30] = 1; means
            //that files 16, 21 and 30 in the list belong to same category that conventionally marked 1. 
            nChart[0] = 0; nChart[4] = 0; nChart[8] = 0; nChart[14] = 0; nChart[15] = 0;
            nChart[16] = 1; nChart[21] = 1; nChart[30] = 1;
            nChart[32] = 2; nChart[34] = 2; nChart[35] = 2; nChart[46] = 2;
            nChart[48] = 3; nChart[51] = 3; nChart[56] = 3; nChart[60] = 3;
            nChart[64] = 4; nChart[70] = 4; nChart[71] = 4; nChart[76] = 4;
            nChart[80] = 5; nChart[82] = 5; nChart[86] = 5; nChart[90] = 5;
            nChart[100] = 6; nChart[105] = 6; nChart[108] = 6;
            nChart[118] = 7; nChart[122] = 7; nChart[123] = 7; nChart[124] = 7;

            string[] folders = { @"C:\APOLAR\PATENTCORPUS128" };
            int numberOfExpectedClusters = 8;
            //Expected result. It is needed only for computation of the accuracy.
            int counter = 0;
            int[] nExpected = new int[128];
            for (int i = 0; i < 8; ++i)
            {
                for (int j = 0; j < 16; ++j)
                {
                    nExpected[counter++] = i;
                }
            }
            //end

            int[] res = new int[128];
            int[,] matrix = new int[3, 128];  //matrix to contain result from three different methods

            string databaseFolder = "data";
            string processedFilesList = "ProcessedFilesList.txt";
            string documentTermMatrix = "DocWordMatrix.dat";

            string path = Path.GetDirectoryName(Assembly.GetExecutingAssembly().Location);
            databaseFolder = Path.Combine(path, databaseFolder);
            processedFilesList = Path.Combine(databaseFolder, processedFilesList);
            documentTermMatrix = Path.Combine(databaseFolder, documentTermMatrix);

            if (!Helper.VerifyFiles(databaseFolder, processedFilesList, documentTermMatrix, nChart))
            {
                Console.WriteLine("Data files are either absent or misformatted. Run Split and Merge to create them.");
                return;
            }

            //1. The following is test for Naive Bayes
            NB nb = new NB();
            if (!nb.processNB(processedFilesList, documentTermMatrix, numberOfExpectedClusters))
            {
                Console.WriteLine("Failed to process Naive Bayes");
            }
            else
            {
                nb.showFilesInClusters();
                Console.WriteLine("Accuracy for NB {0}", nb.computeF_measure(nExpected, numberOfExpectedClusters));
                Console.WriteLine();
                res = nb.getResult();
                for (int i = 0; i < res.Length; ++i)
                {
                    matrix[0, i] = res[i];
                }
            }
            //End of Naive Bayes

            //2. Next is test for clustering by orthogonal decomposition invented by Andrew Polar. 
            Orthogon ortho = new Orthogon();
            ortho.BuildOrthogonalClassifier(documentTermMatrix, processedFilesList);
            ortho.ClusterDocuments(documentTermMatrix);
            Console.WriteLine("Accuracy for PO {0}", ortho.computeF_measure(nExpected, numberOfExpectedClusters));
            Console.WriteLine();
            res = ortho.getResult();
            for (int i = 0; i < res.Length; ++i)
            {
                matrix[1, i] = res[i];
            }
            //End of orthogonal decomposition 

            //3. Next is Random Forest with orthogonalization decomposition invented by Andrew Polar. 
            RFPlusPO rf = new RFPlusPO();
            if (!rf.buildTrainingVectors(documentTermMatrix, processedFilesList))
            {
                Console.WriteLine("Failed to build training matrix");
            }
            else
            {
                rf.buidForest(50);
                rf.processVectors(documentTermMatrix, numberOfExpectedClusters);
                Console.WriteLine("Accuracy for RF {0}", rf.computeF_measure(nExpected, numberOfExpectedClusters));
                Console.WriteLine();
                res = rf.getResult();
                for (int i = 0; i < res.Length; ++i)
                {
                    matrix[2, i] = res[i];
                }
            }
            //End of RF

            //This is utilization of several different technologies, getting best result from simple voting
            Console.WriteLine("Result for Triple Clustering\n");
            for (int i = 0; i < 128; ++i)
            {
                Console.Write(" {0:D} - ", i);
                for (int j = 0; j < 3; ++j)
                {
                    Console.Write(" {0:D} ", matrix[j, i]);
                }
                Console.Write(" --expected-- {0:D}", nExpected[i]);
                Console.WriteLine();
            }
            Console.WriteLine();
            Console.WriteLine("At this point user can manually examine documents ");
            Console.WriteLine("where different methods disagree and add them to training sample.");
        }
    }
}
