﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace helper
{
    public class Row
    {
        public int nClusterIndex = -1;
        public List<int> m_files = new List<int>(0);
        public List<int> m_data = new List<int>(0);
        public List<int> m_pos = new List<int>(0);
        public double norm = 0.0;
    }

    static class Helper
    {
        private static int[] m_nTabooList = null;
        private static int[] m_files = null;
        private static string[] m_fileName = null;
        static int[] m_lookup = null;

        private static bool isFormatted(string processedFilesList)
        {
            StreamReader inStream = new StreamReader(processedFilesList);
            string buffer = inStream.ReadLine();
            if (buffer == null) return false;
            inStream.Close();
            string[] s = buffer.Split(',');
            if (s.Length != 3) return false;
            return true;
        }

        public static bool VerifyFiles(string databaseFolder, string processedFilesList, string documentTermMatrix, int[] nChart)
        {
            DirectoryInfo di = new DirectoryInfo(databaseFolder);
            if (!di.Exists) return false;
            if (!File.Exists(processedFilesList)) return false;
            if (!File.Exists(documentTermMatrix)) return false;

            string tmpFile = Path.Combine(databaseFolder, "tmp.txt");
            if (File.Exists(tmpFile)) File.Delete(tmpFile);

            StreamWriter outStream = new StreamWriter(tmpFile);
            StreamReader inStream = new StreamReader(processedFilesList);
            int cnt = 0;
            while (true)
            {
                string buffer = inStream.ReadLine();
                if (buffer == null) break;
                else
                {
                    if (cnt < nChart.Length)
                    {
                        string[] s = buffer.Split(',');
                        if (s.Length == 2)
                        {
                            outStream.WriteLine(String.Format("{0}, {1}, {2}", s[0], nChart[cnt], s[1]));

                        }
                        else if (s.Length == 3)
                        {
                            outStream.WriteLine(String.Format("{0}, {1}, {2}", s[0], nChart[cnt], s[2]));
                        }
                        else
                        {
                            inStream.Close();
                            outStream.Close();
                            return false;
                        }
                    }
                    ++cnt;
                }
            }
            inStream.Close();
            outStream.Flush();
            outStream.Close();

            File.Replace(tmpFile, processedFilesList, null);
            File.Delete(tmpFile);

            return true;
        }

        private static bool parseFileListString(string s, ref int index, ref int taboo, ref string name)
        {
            if (s == null) return false;
            string[] local = s.Split(',');
            if (local == null) return false;
            if (local.Length < 3) return false;

            try
            {
                index = Convert.ToInt32(local[0].Trim());
                taboo = Convert.ToInt32(local[1].Trim());
                name = local[2].Trim();
            }
            catch (Exception e)
            {
                Console.WriteLine(e.Message);
                Environment.Exit(1);
            }

            return true;
        }

        private static bool populateFileAndTabooLists(string listFiles)
        {
            if (!File.Exists(listFiles)) return false;

            int index = 0;
            int taboo = -1;
            string name = "";
            List<int> listIndex = new List<int>();
            List<int> listTaboo = new List<int>();
            List<string> listName = new List<string>();
            StreamReader fileListStream = new StreamReader(listFiles);
            string s = string.Empty;
            while (s != null)
            {
                s = fileListStream.ReadLine();
                if (!parseFileListString(s, ref index, ref taboo, ref name))
                {
                    break;
                }
                else
                {
                    listIndex.Add(index);
                    listTaboo.Add(taboo);
                    listName.Add(name);
                }
            }
            fileListStream.Close();

            int size = listIndex.Count;
            m_files = new int[size];
            m_nTabooList = new int[size];
            m_fileName = new string[size];

            for (int i = 0; i < size; ++i)
            {
                m_files[i] = listIndex[i];
                m_nTabooList[i] = listTaboo[i];
                m_fileName[i] = listName[i];
            }
            listIndex.Clear();
            listTaboo.Clear();
            listName.Clear();

            return true;
        }

                public static int[] getTabooList(string listFiles)
        {
            if (m_nTabooList == null)
            {
                populateFileAndTabooLists(listFiles);
            }
            return m_nTabooList;
        }

        public static int[] getFileIndex(string listFiles)
        {
            if (m_files == null)
            {
                populateFileAndTabooLists(listFiles);
            }
            return m_files;
        }

        public static string[] getFileNames(string listFiles)
        {
            if (m_fileName == null)
            {
                populateFileAndTabooLists(listFiles);
            }
            return m_fileName;
        }

        static void GetMaxMatrixElement(double[,] matrix, int size, ref int row, ref int col)
        {
            row = 0;
            col = 0;
            double max = matrix[row, col];
            for (int i = 0; i < size; ++i)
            {
                for (int j = 0; j < size; ++j)
                {
                    if (matrix[i, j] > max)
                    {
                        row = i;
                        col = j;
                        max = matrix[i, j];
                    }
                }
            }
        }

        static double formula(double recall, double precision)
        {
            if (recall <= 0.0 && precision <= 0.0) return 0.0;
            else return (2.0 * recall * precision / (recall + precision));
        }

        //function source
        //http://bytes.com/topic/c-sharp/answers/628034-number-formatting-leading-spaces
        public static string formatInt(int intVal, int intTotalsize)
        {
            string retVal = "";
            if (intVal.ToString().Length < intTotalsize)
            {
                for (int slice = 0; slice < intTotalsize - intVal.ToString().Length; slice++)
                {
                    retVal = retVal + " ";
                }
                retVal = retVal + intVal.ToString();
            }
            else
            {
                retVal = intVal.ToString();
            }
            return retVal;
        }

        static public double Compute_F_Measure(int[] expectedCats, int[] actualCats, bool showMatrix, string legend)
        {
            if (legend == null) legend = "legend";
            double f_measure = -1.0;

            //Verification of passed data.
            if (expectedCats.Length != actualCats.Length)
            {
                Console.WriteLine("Error, the sizes of expected and actual data arrays mismatch.");
                return f_measure;
            }
            int min = expectedCats[0];
            int max = expectedCats[0];
            foreach (int x in expectedCats)
            {
                if (x < min) min = x;
                if (x > max) max = x;
            }
            if (min != 0)
            {
                Console.WriteLine("Error, minimum value of expected category should be 0.");
                return f_measure;
            }
            int[] contentTest = new int[max + 1];
            for (int i = 0; i < max + 1; ++i)
            {
                contentTest[i] = 0;
            }
            foreach (int x in expectedCats)
            {
                ++contentTest[x];
            }
            foreach (int x in contentTest)
            {
                if (x == 0)
                {
                    Console.WriteLine("Error, expected categories should be in contiguous range 0 to max.");
                    return f_measure;
                }
            }
            foreach (int x in actualCats)
            {
                if (x < min || x > max)
                {
                    Console.WriteLine("Error, range for actual categories is different from expected.");
                    return f_measure;
                }
            }
            //End of verification of passed data.

            int size = max + 1;
            double[,] matrix = new double[size, size];
            for (int i = 0; i < size; ++i)
            {
                for (int j = 0; j < size; ++j)
                {
                    matrix[i, j] = 0.0;
                }
            }

            for (int i = 0; i < size; ++i)
            {
                for (int j = 0; j < expectedCats.Length; ++j)
                {
                    if (expectedCats[j] == i)
                    {
                        matrix[i, actualCats[j]] += 1.0;
                    }
                }
            }

            double[] totalInRows = new double[size];
            double[] totalInCols = new double[size];
            for (int i = 0; i < size; ++i)
            {
                totalInRows[i] = 0.0;
                totalInCols[i] = 0.0;
            }

            for (int i = 0; i < size; ++i)
            {
                for (int j = 0; j < size; ++j)
                {
                    totalInRows[i] += matrix[i, j];
                }
            }

            for (int j = 0; j < size; ++j)
            {
                for (int i = 0; i < size; ++i)
                {
                    totalInCols[j] += matrix[i, j];
                }
            }

            for (int i = 0; i < size; ++i)
            {
                for (int j = 0; j < size; ++j)
                {
                    double recall = 0.0;
                    double precision = 0.0;
                    if (totalInRows[i] > 0.0) recall = matrix[i, j] / totalInRows[i];
                    if (totalInCols[j] > 0.0) precision = matrix[i, j] / totalInCols[j];
                    matrix[i, j] = formula(recall, precision);
                }
            }

            m_lookup = null;
            m_lookup = new int[size];
            double[] fm = new double[size];

            for (int counter = 0; counter < size; ++counter)
            {
                int row = -1;
                int col = -1;
                GetMaxMatrixElement(matrix, size, ref row, ref col);
                m_lookup[col] = row;
                fm[row] = matrix[row, col];
                for (int i = 0; i < size; ++i) matrix[row, i] = -1.0;
                for (int j = 0; j < size; ++j) matrix[j, col] = -1.0;
            }

            for (int i = 0; i < actualCats.Length; ++i)
            {
                actualCats[i] = m_lookup[actualCats[i]];
            }

            f_measure = 0.0;
            for (int i = 0; i < size; ++i)
            {
                f_measure += fm[i];
            }
            f_measure /= (double)(size);

            if (showMatrix)
            {
                for (int i = 0; i < size; ++i)
                {
                    for (int j = 0; j < size; ++j)
                    {
                        matrix[i, j] = 0.0;
                    }
                }

                for (int i = 0; i < size; ++i)
                {
                    for (int j = 0; j < expectedCats.Length; ++j)
                    {
                        if (expectedCats[j] == i)
                        {
                            matrix[i, actualCats[j]] += 1.0;
                        }
                    }
                }
                Console.WriteLine("{0} {1}\n", "The matrix of actual results for ", legend);
                for (int i = 0; i < size; ++i)
                {
                    for (int j = 0; j < size; ++j)
                    {
                        Console.Write("{0}", formatInt((int)(matrix[i, j]), 4)); 
                    }
                    Console.WriteLine();
                }
                Console.WriteLine();
            }

            return f_measure;
        }
    }
}
