﻿/*
   Copyright 2012, Andrew Polar

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using helper;

namespace tripleclustering
{
    class Orthogon
    {
        double[,] m_classifier = null;
        int m_nRows;
        int m_nCols;
        List<int> m_categories = new List<int>();

        public Orthogon()
        {
        }

        private void Orthoginalization()
        {
            double norm = 0.0;
            for (int i = 0; i < m_nRows; ++i)
            {
                for (int j = 0; j < m_nCols; ++j)
                {
                    norm += m_classifier[i, j] * m_classifier[i, j];
                }
            }
            norm = Math.Sqrt(norm);
            for (int i = 0; i < m_nRows; ++i)
            {
                for (int j = 0; j < m_nCols; ++j)
                {
                    if (m_classifier[i, j] > 0.0)
                    {
                        m_classifier[i, j] /= norm;
                    }
                }
            }
            double[,] tmpSym = new double[m_nRows, m_nRows];
            double[,] tmpRec = new double[m_nRows, m_nCols];
            int counter = 0;
            while (true)
            {
                for (int i = 0; i < m_nRows; ++i)
                {
                    for (int j = i; j < m_nRows; ++j)
                    {
                        tmpSym[i, j] = 0.0;
                        for (int k = 0; k < m_nCols; ++k)
                        {
                            tmpSym[i, j] += m_classifier[i, k] * m_classifier[j, k];
                        }
                    }
                }
                for (int i = 1; i < m_nRows; ++i)
                {
                    for (int j = 0; j < i; ++j)
                    {
                        tmpSym[i, j] = tmpSym[j, i];
                    }
                }

                for (int i = 0; i < m_nRows; ++i)
                {
                    for (int j = 0; j < m_nCols; ++j)
                    {
                        tmpRec[i, j] = m_classifier[i, j];
                    }
                }

                for (int i = 0; i < m_nRows; ++i)
                {
                    for (int j = 0; j < m_nCols; ++j)
                    {
                        m_classifier[i, j] = 0.0;
                        for (int k = 0; k < m_nRows; ++k)
                        {
                            m_classifier[i, j] += tmpSym[i, k] * tmpRec[k, j];
                        }

                        m_classifier[i, j] = 1.5 * tmpRec[i, j] - 0.5 * m_classifier[i, j];
                    }
                }

                double side = 0.0;
                double trace = 0.0;
                for (int i = 0; i < m_nRows; ++i)
                {
                    for (int j = 0; j < m_nRows; ++j)
                    {
                        if (i == j)
                        {
                            trace += tmpSym[i, j];
                        }
                        else
                        {
                            side += tmpSym[i, j];
                        }
                    }
                }

                if (side / trace < 0.00001) break;
                if (++counter >= 100) break;
            }
        }

        public void BuildOrthogonalClassifier(string dataFile, string filesList)
        {
            int[] row2category = Helper.getTabooList(filesList);
            m_nRows = -1;
            foreach (int x in row2category)
            {
                if (x > m_nRows) m_nRows = x;
            }
            ++m_nRows;

            m_nCols = -1;
            FileInfo fi = new FileInfo(dataFile);
            long nBytes = fi.Length;
            int nNonZeros = (int)(nBytes / 10);
            using (FileStream stream = new FileStream(dataFile, FileMode.Open))
            {
                using (BinaryReader reader = new BinaryReader(stream))
                {
                    for (long i = 0; i < nNonZeros; ++i)
                    {
                        int nR = reader.ReadInt32();
                        int nC = reader.ReadInt32();
                        short sum = reader.ReadInt16();
                        if (nC > m_nCols) m_nCols = nC;
                    }
                    reader.Close();
                }
                stream.Close();
            }
            ++m_nCols;

            //Allocate memory
            m_classifier = new double[m_nRows, m_nCols];
            for (int x = 0; x < m_nRows; ++x)
            {
                for (int y = 0; y < m_nCols; ++y)
                {
                    m_classifier[x, y] = 0.0;
                }
            }

            //Populate matrix
            using (FileStream stream = new FileStream(dataFile, FileMode.Open))
            {
                using (BinaryReader reader = new BinaryReader(stream))
                {
                    for (long i = 0; i < nNonZeros; ++i)
                    {
                        int nR = reader.ReadInt32();
                        int nC = reader.ReadInt32();
                        short sum = reader.ReadInt16();

                        int row = row2category[nR];
                        if (row >= 0)
                        {
                            m_classifier[row, nC] += (double)(sum);
                        }

                    }
                    reader.Close();
                }
                stream.Close();
            }

            //Orthogonalization of the matrix
            Orthoginalization();
        }

        int getCategory(List<int> pos, List<int> val)
        {
            double[] catVector = new double[m_nRows];
            for (int i = 0; i < m_nRows; ++i)
            {
                catVector[i] = 0.0;
                int k = 0;
                foreach (int position in pos)
                {
                    catVector[i] += m_classifier[i, position] * val[k];
                    ++k;
                }
            }

            int maxPos = 0;
            double max = catVector[maxPos];
            for (int i = 0; i < m_nRows; ++i)
            {
                if (catVector[i] > max)
                {
                    maxPos = i;
                    max = catVector[maxPos];
                }
            }

            return maxPos;
        }

        public void ClusterDocuments(string dataFile)
        {
            FileInfo fi = new FileInfo(dataFile);
            long nBytes = fi.Length;
            int nNonZeros = (int)(nBytes / 10);

            int category;
            int nRow = -1;
            int nCol = -1;
            int nPrevRow = 0;
            List<int> pos = new List<int>();
            List<int> val = new List<int>();
            m_categories.Clear();
            using (FileStream stream = new FileStream(dataFile, FileMode.Open))
            {
                using (BinaryReader reader = new BinaryReader(stream))
                {
                    for (long i = 0; i < nNonZeros; ++i)
                    {
                        nRow = reader.ReadInt32();
                        nCol = reader.ReadInt32();
                        short sum = reader.ReadInt16();

                        if (nRow != nPrevRow)
                        {
                            category = getCategory(pos, val);
                            m_categories.Add(category);

                            pos.Clear();
                            val.Clear();
                            nPrevRow = nRow;
                        }
                        else
                        {
                            pos.Add(nCol);
                            val.Add(sum);
                        }
                    }
                    reader.Close();
                }
                stream.Close();
            }

            category = getCategory(pos, val);
            m_categories.Add(category);
        }

        public double computeF_measure(int[] nExpected, int nCategories)
        {
            return Helper.Compute_F_Measure(nExpected, m_categories.ToArray(), true, "PO");
        }

        public int[] getResult()
        {
            return m_categories.ToArray();
        }
    }
}
