﻿/*
   Copyright 2012, Andrew Polar

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace wordprocessing
{
    class SimpleFileFinder
    {
        private string[] m_folders;
        private string[] m_masks;
        public List<string> m_foundFiles = new List<string>();

        public SimpleFileFinder(string[] folders, string allMasks)
        {
            m_folders = folders;
            m_masks = allMasks.Split('|');
            for (int i = 0; i < m_masks.Length; ++i)
            {
                m_masks[i] = m_masks[i].Trim();
            }
            m_foundFiles.Clear();
        }

        private void WalkDirectoryTree(System.IO.DirectoryInfo root, string[] masks)
        {
            try
            {
                System.IO.DirectoryInfo[] subDirs = root.GetDirectories();
                foreach (System.IO.DirectoryInfo dirInfo in subDirs)
                {
                    WalkDirectoryTree(dirInfo, masks);
                }

                foreach (string mask in masks)
                {
                    System.IO.FileInfo[] files = root.GetFiles(mask);
                    foreach (FileInfo fi in files)
                    {
                        if (!m_foundFiles.Contains(fi.FullName))
                        {
                            m_foundFiles.Add(fi.FullName);
                        }
                    }
                }
            }
            catch (UnauthorizedAccessException e)
            {
                //inaccessible folder can be logged
                //MessageBox.Show(e.Message, "Access error", MessageBoxButtons.OK);
            }
            catch (System.IO.DirectoryNotFoundException e)
            {
                //inaccessible folder can be logged
                //MessageBox.Show(e.Message, "Access error", MessageBoxButtons.OK);
            }
        }

        public void EnumerateFiles()
        {
            foreach (string folder in m_folders)
            {
                DirectoryInfo di = new DirectoryInfo(folder);
                WalkDirectoryTree(di, m_masks);
            }
        }

        public int GetNumberOfFiles()
        {
            return m_foundFiles.Count;
        }
    }
}
